function acc_cheat=doubleDippingDemo(D,Dsel,N)
% function acc_cheat=doubleDippingDemo(D,Dsel,N)
% 
% Demonstrates the implications of selecting features outside of
% cross-validation, by generating random samples and being able to classify
% with more than chance accuracy.
%
% The problem grows worse with increasing dimensionality, as there is more
% probability of finding chance correlations. It is decreased by using more
% samples. In any case feature selection without dataset splitting is a bad
% idea.
%
% This is a classic in statistics and pattern recognition. For a good
% discussion in Neuroimaging problems, see
% Kriegeskorte et al.,  Circular analysis in systems neuroscience: the
% dangers of double dipping, Nature Neuroscience 12, 2009
%
% EXAMPLE USE
%   Say we have 1000 voxels, we want to select the 10 most predictive,
%   and we have 50 subjects (25 controls, 25 patients). This
%   acc=doubleDippingDemo(1000,10,50);
%
% IN:
%   D: a scalar - the dimensionality of the feature space to generate
%   Dsel: a scalar - how many feature to select
%   N: a scalar - the number of samples to generate (eg. number of
%       subjects)
%
% OUT:
%   acc_cheat: a scalar between 0 and 1 - classification accuracy with
%       cheating
% 
% v1.0 May 2010 Jonas Richiardi, Ecole Polytechnique Federale de Lausanne
% - initial release for Advanced fMRI course (Lemanic Neurosciences Ph.D.)

% minimal sanity check
if N < 4
    error('Must have a least 4 samples');
end
% check for the classifier availability
if ~(license('test','Statistics_Toolbox') && exist('NaiveBayes','class'))
    error('You must have the statistics toolbox R2009a or above.');
    % or just hack the corresponding line to substitute your favourite
    % learning machine
end

% generate data and class labels
X=randn(D,N);                                   % random normal data
Y=[zeros(1,floor(N/2)) ones(1,ceil(N/2))];      % class labels (balanced)

% select over all data set - very wrong!
myBF=selectFeatures(X,Y,Dsel,1);                % myBF: index vector

% then split 50:50 in training:testing
idx_TR=[1:floor(N/4) floor((N/2+1)):floor((3*N/4))];
idx_TE=[floor((N/4+1)):floor(N/2) floor((3*N/4+1)):N];    
X_TR=X(:,idx_TR);
X_TE=X(:,idx_TE);
Y_TR=Y(idx_TR);                                 % likewise for class labels
Y_TE=Y(idx_TE);

% train classifier on train set
myCls = NaiveBayes.fit(X_TR(myBF(1:Dsel),:)', Y_TR);
% test classifier on test set
predLabels = predict(myCls,X_TE(myBF(1:Dsel),:)');

% look at the wildly optimistic error rate
correct=predLabels(:)==Y_TE(:);
acc_cheat=sum(correct)/length(Y_TE);
disp(['Cheating accuracy: ' num2str(acc_cheat,'%1.2f')])
